require "active_model"
require "yaml"
require_relative "../api"

module OneEye
  class Base
    include ActiveModel::Serializers::JSON

    def self.api
      OneEye::API.new
    end

    def initialize(attributes = {})
      self.attributes = attributes
      defaults
    end

    def defaults
      <<-eos
        Please define behavior in subclasses.
        See: Template method pattern.
        http://en.wikipedia.org/wiki/Template_method_pattern
      eos
    end

    def attributes=(attrs)
      make_array_convertable attrs if attrs.is_a? Array
      attrs.to_hash.each { |k, v| send "#{k.to_s.gsub(/\-/) {}}=", v }
    end

    def attributes
      instance_variables.inject Hash.new do |attributes, key|
        key = key.to_s.gsub(/\@/) { "" }
        attributes[key] = send key
        attributes
      end
    end

    def self.find(attributes={})
      new(api.get(resources_url, attributes))
    end

    def self.where(attributes={})
      api.get(resources_url, attributes).map do |model|
        new model
      end
    end

    def self.create(attributes={})
      new(api.post(resources_url, attributes))
    end

    def update(attributes={})
      self.attributes = attributes
      self.attributes = api.put(resources_url, self.attributes)
      self
    end

    def destroy
      api.delete resources_url, attributes
    end

    def delete
      destroy
    end

  private
    def self.resources_url
      self.name.sub(/\w+\:\:/) {}.underscore.downcase.pluralize
    end

    def self.method_missing(name, *args, &block)
      missing_finder name, *args, &block
    end

    def method_missing(name, *args, &block)
      if name[-1] == "="
        return instance_variable_set "@#{name[0..-2]}", args[0]
      else
        instance_variable_get "@#{name}"
      end
    end

    def api
      self.class.api
    end

    def resources_url
      self.class.resources_url
    end

    def self.missing_finder(name, *args, &block)
      @models = api.get("#{self.name.gsub(/OneEye\:\:/) {}.underscore}/#{name.to_s.underscore}", *args)
      return @models.map { |model| new model } if @models.map
      new @models
    end

    def make_array_convertable(attrs)
      def attrs.to_hash
        Hash[self.each_slice(2).to_a]
      end
    end
  end
end
